@@ -2,14 +2,16 @@
 //
 // GList.h
 //
-// Copyright 2001-2002 Glyph & Cog, LLC
+// Copyright 2001-2003 Glyph & Cog, LLC
 //
 //========================================================================
 
 #ifndef GLIST_H
 #define GLIST_H
 
-#ifdef __GNUC__
+#include <aconf.h>
+
+#ifdef USE_GCC_PRAGMAS
 #pragma interface
 #endif
 
@@ -36,12 +38,19 @@ public:
   // Get the number of elements.
   int getLength() { return length; }
 
+  // Returns a (shallow) copy of this list.
+  GList *copy();
+
   //----- ordered list support
 
   // Return the <i>th element.
   // Assumes 0 <= i < length.
   void *get(int i) { return data[i]; }
 
+  // Replace the <i>th element.
+  // Assumes 0 <= i < length.
+  void put(int i, void *p) { data[i] = p; }
+
   // Append an element to the end of the list.
   void append(void *p);
 
@@ -56,6 +65,14 @@ public:
   // Assumes 0 <= i < length.
   void *del(int i);
 
+  // Sort the list accoring to the given comparison function.
+  // NB: this sorts an array of pointers, so the pointer args need to
+  // be double-dereferenced.
+  void sort(int (*cmp)(const void *ptr1, const void *ptr2));
+
+  // Reverse the list.
+  void reverse();
+
   //----- control
 
   // Set allocation increment to <inc>.  If inc > 0, that many
