@@ -18,6 +18,8 @@
 #include "gtypes.h"
 
 class GString;
+class GList;
+class PDFDoc;
 class XRef;
 class Array;
 class Stream;
@@ -88,6 +90,7 @@ public:
   GfxPattern *lookupPattern(char *name);
   GfxShading *lookupShading(char *name);
   GBool lookupGState(char *name, Object *obj);
+  GBool lookupPropertiesNF(char *name, Object *obj);
 
   GfxResources *getNext() { return next; }
 
@@ -99,9 +102,34 @@ private:
   Object patternDict;
   Object shadingDict;
   Object gStateDict;
+  Object propsDict;
   GfxResources *next;
 };
 
+//------------------------------------------------------------------------
+// GfxMarkedContent
+//------------------------------------------------------------------------
+
+enum GfxMarkedContentKind {
+  gfxMCOptionalContent,
+  gfxMCActualText,
+  gfxMCOther
+};
+
+class GfxMarkedContent {
+public:
+
+  GfxMarkedContent(GfxMarkedContentKind kindA, GBool ocStateA) {
+    kind = kindA;
+    ocState = ocStateA;
+  }
+  ~GfxMarkedContent() {}
+
+  GfxMarkedContentKind kind;
+  GBool ocState;		// true if drawing is enabled, false if
+				//   disabled
+};
+
 //------------------------------------------------------------------------
 // Gfx
 //------------------------------------------------------------------------
@@ -110,14 +138,14 @@ class Gfx {
 public:
 
   // Constructor for regular output.
-  Gfx(XRef *xrefA, OutputDev *outA, int pageNum, Dict *resDict,
+  Gfx(PDFDoc *docA, OutputDev *outA, int pageNum, Dict *resDict,
       double hDPI, double vDPI, PDFRectangle *box,
       PDFRectangle *cropBox, int rotate,
       GBool (*abortCheckCbkA)(void *data) = NULL,
       void *abortCheckCbkDataA = NULL);
 
   // Constructor for a sub-page object.
-  Gfx(XRef *xrefA, OutputDev *outA, Dict *resDict,
+  Gfx(PDFDoc *docA, OutputDev *outA, Dict *resDict,
       PDFRectangle *box, PDFRectangle *cropBox,
       GBool (*abortCheckCbkA)(void *data) = NULL,
       void *abortCheckCbkDataA = NULL);
@@ -141,8 +169,16 @@ public:
   // Get the current graphics state object.
   GfxState *getState() { return state; }
 
+  void drawForm(Object *str, Dict *resDict, double *matrix, double *bbox,
+		GBool transpGroup = gFalse, GBool softMask = gFalse,
+		GfxColorSpace *blendingColorSpace = NULL,
+		GBool isolated = gFalse, GBool knockout = gFalse,
+		GBool alpha = gFalse, Function *transferFunc = NULL,
+		GfxColor *backdropColor = NULL);
+
 private:
 
+  PDFDoc *doc;
   XRef *xref;			// the xref table for this PDF file
   OutputDev *out;		// output device
   GBool subPage;		// is this a sub-page object?
@@ -157,6 +193,12 @@ private:
   double baseMatrix[6];		// default matrix for most recent
 				//   page/form/pattern
   int formDepth;
+  double textClipBBox[4];	// text clipping bounding box
+  GBool textClipBBoxEmpty;	// true if textClipBBox has not been
+				//   initialized yet
+  GBool ocState;		// true if drawing is enabled, false if
+				//   disabled
+  GList *markedContentStack;	// BMC/BDC/EMC stack [GfxMarkedContent]
 
   Parser *parser;		// parser for page content stream(s)
 
@@ -224,10 +266,13 @@ private:
   void opCloseEOFillStroke(Object args[], int numArgs);
   void doPatternFill(GBool eoFill);
   void doPatternStroke();
+  void doPatternText();
+  void doPatternImageMask(Object *ref, Stream *str, int width, int height,
+			  GBool invert, GBool inlineImg);
   void doTilingPatternFill(GfxTilingPattern *tPat,
-			   GBool stroke, GBool eoFill);
+			   GBool stroke, GBool eoFill, GBool text);
   void doShadingPatternFill(GfxShadingPattern *sPat,
-			    GBool stroke, GBool eoFill);
+			    GBool stroke, GBool eoFill, GBool text);
   void opShFill(Object args[], int numArgs);
   void doFunctionShFill(GfxFunctionShading *shading);
   void doFunctionShFill1(GfxFunctionShading *shading,
@@ -274,17 +319,12 @@ private:
   void opMoveSetShowText(Object args[], int numArgs);
   void opShowSpaceText(Object args[], int numArgs);
   void doShowText(GString *s);
+  void doIncCharCount(GString *s);
 
   // XObject operators
   void opXObject(Object args[], int numArgs);
   void doImage(Object *ref, Stream *str, GBool inlineImg);
   void doForm(Object *str);
-  void doForm1(Object *str, Dict *resDict, double *matrix, double *bbox,
-	       GBool transpGroup = gFalse, GBool softMask = gFalse,
-	       GfxColorSpace *blendingColorSpace = NULL,
-	       GBool isolated = gFalse, GBool knockout = gFalse,
-	       GBool alpha = gFalse, Function *transferFunc = NULL,
-	       GfxColor *backdropColor = NULL);
 
   // in-line image operators
   void opBeginImage(Object args[], int numArgs);
@@ -305,6 +345,8 @@ private:
   void opEndMarkedContent(Object args[], int numArgs);
   void opMarkPoint(Object args[], int numArgs);
 
+  GfxState *saveStateStack();
+  void restoreStateStack(GfxState *oldState);
   void pushResources(Dict *resDict);
   void popResources();
 };
