@@ -23,8 +23,9 @@
 class Dict;
 class CMap;
 class CharCodeToUnicode;
-class TrueTypeFontFile;
+class FoFiTrueType;
 struct GfxFontCIDWidths;
+struct Base14FontMapEntry;
 
 //------------------------------------------------------------------------
 // GfxFontType
@@ -35,12 +36,16 @@ enum GfxFontType {
   fontUnknownType,
   fontType1,
   fontType1C,
+  fontType1COT,
   fontType3,
   fontTrueType,
+  fontTrueTypeOT,
   //----- GfxCIDFont
   fontCIDType0,
   fontCIDType0C,
-  fontCIDType2
+  fontCIDType0COT,
+  fontCIDType2,
+  fontCIDType2OT
 };
 
 //------------------------------------------------------------------------
@@ -71,6 +76,41 @@ struct GfxFontCIDWidths {
   int nExcepsV;			// number of valid entries in excepsV
 };
 
+//------------------------------------------------------------------------
+// GfxFontLoc
+//------------------------------------------------------------------------
+
+enum GfxFontLocType {
+  gfxFontLocEmbedded,		// font embedded in PDF file
+  gfxFontLocExternal,		// external font file
+  gfxFontLocResident		// font resident in PS printer
+};
+
+class GfxFontLoc {
+public:
+
+  GfxFontLoc();
+  ~GfxFontLoc();
+
+  GfxFontLocType locType;
+  GfxFontType fontType;
+  Ref embFontID;		// embedded stream obj ID
+				//   (if locType == gfxFontLocEmbedded)
+  GString *path;		// font file path
+				//   (if locType == gfxFontLocExternal)
+				// PS font name
+				//   (if locType == gfxFontLocResident)
+  int fontNum;			// for TrueType collections
+				//   (if locType == gfxFontLocExternal)
+  GString *encoding;		// PS font encoding, only for 16-bit fonts
+				//   (if locType == gfxFontLocResident)
+  int wMode;			// writing mode, only for 16-bit fonts
+				//   (if locType == gfxFontLocResident)
+  int substIdx;			// substitute font index
+				//   (if locType == gfxFontLocExternal,
+				//   and a Base-14 substitution was made)
+};
+
 //------------------------------------------------------------------------
 // GfxFont
 //------------------------------------------------------------------------
@@ -87,7 +127,8 @@ public:
   // Build a GfxFont object.
   static GfxFont *makeFont(XRef *xref, char *tagA, Ref idA, Dict *fontDict);
 
-  GfxFont(char *tagA, Ref idA, GString *nameA);
+  GfxFont(char *tagA, Ref idA, GString *nameA,
+	  GfxFontType typeA, Ref embFontIDA);
 
   virtual ~GfxFont();
 
@@ -102,12 +143,9 @@ public:
   // Does this font match the tag?
   GBool matches(char *tagA) { return !tag->cmp(tagA); }
 
-  // Get base font name.
-  GString *getName() { return name; }
-
   // Get the original font name (ignornig any munging that might have
   // been done to map to a canonical Base-14 font name).
-  GString *getOrigName() { return origName; }
+  GString *getName() { return name; }
 
   // Get font type.
   GfxFontType getType() { return type; }
@@ -122,11 +160,8 @@ public:
   // NULL if there is no embedded font.
   GString *getEmbeddedFontName() { return embFontName; }
 
-  // Get the name of the external font file.  Returns NULL if there
-  // is no external font file.
-  GString *getExtFontFile() { return extFontFile; }
-
   // Get font descriptor flags.
+  int getFlags() { return flags; }
   GBool isFixedWidth() { return flags & fontFixedWidth; }
   GBool isSerif() { return flags & fontSerif; }
   GBool isSymbolic() { return flags & fontSymbolic; }
@@ -146,8 +181,14 @@ public:
   // Return the writing mode (0=horizontal, 1=vertical).
   virtual int getWMode() { return 0; }
 
-  // Read an external or embedded font file into a buffer.
-  char *readExtFontFile(int *len);
+  // Locate the font file for this font.  If <ps> is true, includes PS
+  // printer-resident fonts.  Returns NULL on failure.
+  GfxFontLoc *locateFont(XRef *xref, GBool ps);
+
+  // Locate a Base-14 font file for a specified font name.
+  static GfxFontLoc *locateBase14Font(GString *base14Name);
+
+  // Read an embedded font file into a buffer.
   char *readEmbFontFile(XRef *xref, int *len);
 
   // Get the next char from a string <s> of <len> bytes, returning the
@@ -162,22 +203,21 @@ public:
 
 protected:
 
+  static GfxFontType getFontType(XRef *xref, Dict *fontDict, Ref *embID);
   void readFontDescriptor(XRef *xref, Dict *fontDict);
   CharCodeToUnicode *readToUnicodeCMap(Dict *fontDict, int nBits,
 				       CharCodeToUnicode *ctu);
-  void findExtFontFile();
+  static GfxFontLoc *getExternalFont(GString *path, GBool cid);
 
   GString *tag;			// PDF font tag
   Ref id;			// reference (used as unique ID)
   GString *name;		// font name
-  GString *origName;		// original font name
   GfxFontType type;		// type of font
   int flags;			// font descriptor flags
   GString *embFontName;		// name of embedded font
   Ref embFontID;		// ref to embedded font file stream
-  GString *extFontFile;		// external font file name
-  double fontMat[6];		// font matrix (Type 3 only)
-  double fontBBox[4];		// font bounding box (Type 3 only)
+  double fontMat[6];		// font matrix
+  double fontBBox[4];		// font bounding box
   double missingWidth;		// "default" width
   double ascent;		// max height above baseline
   double descent;		// max depth below baseline
@@ -192,7 +232,7 @@ class Gfx8BitFont: public GfxFont {
 public:
 
   Gfx8BitFont(XRef *xref, char *tagA, Ref idA, GString *nameA,
-	      GfxFontType typeA, Dict *fontDict);
+	      GfxFontType typeA, Ref embFontIDA, Dict *fontDict);
 
   virtual ~Gfx8BitFont();
 
@@ -220,7 +260,7 @@ public:
 
   // Return a char code-to-GID mapping for the provided font file.
   // (This is only useful for TrueType fonts.)
-  Gushort *getCodeToGIDMap(TrueTypeFontFile *ff);
+  int *getCodeToGIDMap(FoFiTrueType *ff);
 
   // Return the Type 3 CharProc dictionary, or NULL if none.
   Dict *getCharProcs();
@@ -233,6 +273,7 @@ public:
 
 private:
 
+  Base14FontMapEntry *base14;	// for Base-14 fonts only; NULL otherwise
   char *enc[256];		// char code --> char name
   char encFree[256];		// boolean for each char name: if set,
 				//   the string is malloc'ed
@@ -242,6 +283,8 @@ private:
   double widths[256];		// character widths
   Object charProcs;		// Type 3 CharProcs dictionary
   Object resources;		// Type 3 Resources dictionary
+
+  friend class GfxFont;
 };
 
 //------------------------------------------------------------------------
@@ -252,7 +295,7 @@ class GfxCIDFont: public GfxFont {
 public:
 
   GfxCIDFont(XRef *xref, char *tagA, Ref idA, GString *nameA,
-	     Dict *fontDict);
+	     GfxFontType typeA, Ref embFontIDA, Dict *fontDict);
 
   virtual ~GfxCIDFont();
 
@@ -273,15 +316,18 @@ public:
 
   // Return the CID-to-GID mapping table.  These should only be called
   // if type is fontCIDType2.
-  Gushort *getCIDToGID() { return cidToGID; }
+  int *getCIDToGID() { return cidToGID; }
   int getCIDToGIDLen() { return cidToGIDLen; }
 
 private:
 
+  GString *collection;		// collection name
   CMap *cMap;			// char code --> CID
-  CharCodeToUnicode *ctu;	// CID --> Unicode
+  CharCodeToUnicode *ctu;	// CID/char code --> Unicode
+  GBool ctuUsesCharCode;	// true: ctu maps char code to Unicode;
+				//   false: ctu maps CID to Unicode
   GfxFontCIDWidths widths;	// character widths
-  Gushort *cidToGID;		// CID --> GID mapping (for embedded
+  int *cidToGID;		// CID --> GID mapping (for embedded
 				//   TrueType fonts)
   int cidToGIDLen;
 };
