@@ -65,6 +65,7 @@ void OutputDev::updateAll(GfxState *state) {
   updateStrokeOpacity(state);
   updateFillOverprint(state);
   updateStrokeOverprint(state);
+  updateOverprintMode(state);
   updateTransfer(state);
   updateFont(state);
 }
@@ -89,6 +90,13 @@ void OutputDev::drawImageMask(GfxState *state, Object *ref, Stream *str,
   }
 }
 
+void OutputDev::setSoftMaskFromImageMask(GfxState *state,
+					 Object *ref, Stream *str,
+					 int width, int height, GBool invert,
+					 GBool inlineImg) {
+  drawImageMask(state, ref, str, width, height, invert, inlineImg);
+}
+
 void OutputDev::drawImage(GfxState *state, Object *ref, Stream *str,
 			  int width, int height, GfxImageColorMap *colorMap,
 			  int *maskColors, GBool inlineImg) {
