@@ -27,6 +27,8 @@ class Links;
 class LinkAction;
 class LinkDest;
 class Outline;
+class OptionalContent;
+class PDFCore;
 
 //------------------------------------------------------------------------
 // PDFDoc
@@ -36,13 +38,13 @@ class PDFDoc {
 public:
 
   PDFDoc(GString *fileNameA, GString *ownerPassword = NULL,
-	 GString *userPassword = NULL, void *guiDataA = NULL);
+	 GString *userPassword = NULL, PDFCore *coreA = NULL);
 #ifdef WIN32
   PDFDoc(wchar_t *fileNameA, int fileNameLen, GString *ownerPassword = NULL,
-	 GString *userPassword = NULL, void *guiDataA = NULL);
+	 GString *userPassword = NULL, PDFCore *coreA = NULL);
 #endif
   PDFDoc(BaseStream *strA, GString *ownerPassword = NULL,
-	 GString *userPassword = NULL, void *guiDataA = NULL);
+	 GString *userPassword = NULL, PDFCore *coreA = NULL);
   ~PDFDoc();
 
   // Was PDF document successfully opened?
@@ -53,6 +55,9 @@ public:
 
   // Get file name.
   GString *getFileName() { return fileName; }
+#ifdef WIN32
+  wchar_t *getFileNameU() { return fileNameU; }
+#endif
 
   // Get the xref table.
   XRef *getXRef() { return xref; }
@@ -86,23 +91,23 @@ public:
   Object *getStructTreeRoot() { return catalog->getStructTreeRoot(); }
 
   // Display a page.
-  void displayPage(OutputDev *out, int page, double hDPI, double vDPI,
-		   int rotate, GBool useMediaBox, GBool crop,
-		   GBool doLinks,
+  void displayPage(OutputDev *out, int page,
+		   double hDPI, double vDPI, int rotate,
+		   GBool useMediaBox, GBool crop, GBool printing,
 		   GBool (*abortCheckCbk)(void *data) = NULL,
 		   void *abortCheckCbkData = NULL);
 
   // Display a range of pages.
   void displayPages(OutputDev *out, int firstPage, int lastPage,
 		    double hDPI, double vDPI, int rotate,
-		    GBool useMediaBox, GBool crop, GBool doLinks,
+		    GBool useMediaBox, GBool crop, GBool printing,
 		    GBool (*abortCheckCbk)(void *data) = NULL,
 		    void *abortCheckCbkData = NULL);
 
   // Display part of a page.
   void displayPageSlice(OutputDev *out, int page,
 			double hDPI, double vDPI, int rotate,
-			GBool useMediaBox, GBool crop, GBool doLinks,
+			GBool useMediaBox, GBool crop, GBool printing,
 			int sliceX, int sliceY, int sliceW, int sliceH,
 			GBool (*abortCheckCbk)(void *data) = NULL,
 			void *abortCheckCbkData = NULL);
@@ -113,18 +118,24 @@ public:
 
   // Returns the links for the current page, transferring ownership to
   // the caller.
-  Links *takeLinks();
+  Links *getLinks(int page);
 
   // Find a named destination.  Returns the link destination, or
   // NULL if <name> is not a destination.
   LinkDest *findDest(GString *name)
     { return catalog->findDest(name); }
 
+  // Process the links for a page.
+  void processLinks(OutputDev *out, int page);
+
 #ifndef DISABLE_OUTLINE
   // Return the outline object.
   Outline *getOutline() { return outline; }
 #endif
 
+  // Return the OptionalContent object.
+  OptionalContent *getOptionalContent() { return optContent; }
+
   // Is the file encrypted?
   GBool isEncrypted() { return xref->isEncrypted(); }
 
@@ -151,29 +162,44 @@ public:
   // Save this file with another name.
   GBool saveAs(GString *name);
 
-  // Return a pointer to the GUI (XPDFCore or WinPDFCore object).
-  void *getGUIData() { return guiData; }
+  // Return a pointer to the PDFCore object.
+  PDFCore *getCore() { return core; }
 
+  // Get the list of embedded files.
+  int getNumEmbeddedFiles() { return catalog->getNumEmbeddedFiles(); }
+  Unicode *getEmbeddedFileName(int idx)
+    { return catalog->getEmbeddedFileName(idx); }
+  int getEmbeddedFileNameLength(int idx)
+    { return catalog->getEmbeddedFileNameLength(idx); }
+  GBool saveEmbeddedFile(int idx, char *path);
+#ifdef WIN32
+  GBool saveEmbeddedFile(int idx, wchar_t *path, int pathLen);
+#endif
+  char *getEmbeddedFileMem(int idx, int *size);
 
 private:
 
   GBool setup(GString *ownerPassword, GString *userPassword);
+  GBool setup2(GString *ownerPassword, GString *userPassword,
+	       GBool repairXRef);
   void checkHeader();
   GBool checkEncryption(GString *ownerPassword, GString *userPassword);
-  void getLinks(Page *page);
+  GBool saveEmbeddedFile2(int idx, FILE *f);
 
   GString *fileName;
+#ifdef WIN32
+  wchar_t *fileNameU;
+#endif
   FILE *file;
   BaseStream *str;
-  void *guiData;
+  PDFCore *core;
   double pdfVersion;
   XRef *xref;
   Catalog *catalog;
-  Links *links;
 #ifndef DISABLE_OUTLINE
   Outline *outline;
 #endif
-
+  OptionalContent *optContent;
 
   GBool ok;
   int errCode;
