@@ -2,14 +2,16 @@
 //
 // Parser.h
 //
-// Copyright 1996-2002 Glyph & Cog, LLC
+// Copyright 1996-2003 Glyph & Cog, LLC
 //
 //========================================================================
 
 #ifndef PARSER_H
 #define PARSER_H
 
-#ifdef __GNUC__
+#include <aconf.h>
+
+#ifdef USE_GCC_PRAGMAS
 #pragma interface
 #endif
 
@@ -23,19 +25,18 @@ class Parser {
 public:
 
   // Constructor.
-  Parser(XRef *xrefA, Lexer *lexerA);
+  Parser(XRef *xrefA, Lexer *lexerA, GBool allowStreamsA);
 
   // Destructor.
   ~Parser();
 
-  // Get the next object from the input stream.
-#ifndef NO_DECRYPTION
-  Object *getObj(Object *obj,
-		 Guchar *fileKey = NULL, int keyLength = 0,
-		 int objNum = 0, int objGen = 0);
-#else
-  Object *getObj(Object *obj);
-#endif
+  // Get the next object from the input stream.  If <simpleOnly> is
+  // true, do not parse compound objects (arrays, dictionaries, or
+  // streams).
+  Object *getObj(Object *obj, GBool simpleOnly = gFalse,
+		 Guchar *fileKey = NULL,
+		 CryptAlgorithm encAlgorithm = cryptRC4, int keyLength = 0,
+		 int objNum = 0, int objGen = 0, int recursion = 0);
 
   // Get stream.
   Stream *getStream() { return lexer->getStream(); }
@@ -47,10 +48,13 @@ private:
 
   XRef *xref;			// the xref table for this PDF file
   Lexer *lexer;			// input stream
+  GBool allowStreams;		// parse stream objects?
   Object buf1, buf2;		// next two tokens
   int inlineImg;		// set when inline image data is encountered
 
-  Stream *makeStream(Object *dict);
+  Stream *makeStream(Object *dict, Guchar *fileKey,
+		     CryptAlgorithm encAlgorithm, int keyLength,
+		     int objNum, int objGen, int recursion);
   void shift();
 };
 
