@@ -2,7 +2,7 @@
 //
 // UTF8.h
 //
-// Copyright 2001 Derek B. Noonburg
+// Copyright 2001-2003 Glyph & Cog, LLC
 //
 //========================================================================
 
@@ -41,3 +41,16 @@ static int mapUTF8(Unicode u, char *buf, int bufSize) {
     return 0;
   }
 }
+
+static int mapUCS2(Unicode u, char *buf, int bufSize) {
+  if (u <= 0xffff) {
+    if (bufSize < 2) {
+      return 0;
+    }
+    buf[0] = (char)((u >> 8) & 0xff);
+    buf[1] = (char)(u & 0xff);
+    return 2;
+  } else {
+    return 0;
+  }
+}
