@@ -2,7 +2,7 @@
 //
 // config.h
 //
-// Copyright 1996-2002 Glyph & Cog, LLC
+// Copyright 1996-2011 Glyph & Cog, LLC
 //
 //========================================================================
 
@@ -14,15 +14,25 @@
 //------------------------------------------------------------------------
 
 // xpdf version
-
-#define xpdfVersion "1.01"
+#define xpdfVersion          "3.03"
+#define xpdfVersionNum       3.03
+#define xpdfMajorVersion     3
+#define xpdfMinorVersion     3
+#define xpdfUpdateVersion    0
+#define xpdfMajorVersionStr  "3"
+#define xpdfMinorVersionStr  "3"
+#define xpdfUpdateVersionStr "0"
 
 // supported PDF version
-#define supportedPDFVersionStr "1.4"
-#define supportedPDFVersionNum 1.4
+#define supportedPDFVersionStr "1.7"
+#define supportedPDFVersionNum 1.7
 
 // copyright notice
-#define xpdfCopyright "Copyright 1996-2002 Glyph & Cog, LLC"
+#define xpdfCopyright "Copyright 1996-2011 Glyph & Cog, LLC"
+
+// Windows resource file stuff
+#define winxpdfVersion "WinXpdf 3.03"
+#define xpdfCopyrightAmp "Copyright 1996-2011 Glyph && Cog, LLC"
 
 //------------------------------------------------------------------------
 // paper size
@@ -42,7 +52,7 @@
 //------------------------------------------------------------------------
 
 // user config file name, relative to the user's home directory
-#if defined(VMS) || (defined(WIN32) && !defined(__CYGWIN32__))
+#if defined(VMS) || defined(WIN32)
 #define xpdfUserConfigFile "xpdfrc"
 #else
 #define xpdfUserConfigFile ".xpdfrc"
@@ -64,55 +74,21 @@
 // default maximum size of color cube to allocate
 #define defaultRGBCube 5
 
-// number of fonts (combined t1lib, FreeType, X server) to cache
-#define xOutFontCacheSize 64
-
-// number of Type 3 fonts to cache
-#define xOutT3FontCacheSize 8
-
 //------------------------------------------------------------------------
 // popen
 //------------------------------------------------------------------------
 
-#ifdef _MSC_VER
+#if defined(_MSC_VER) || defined(__BORLANDC__)
 #define popen _popen
 #define pclose _pclose
 #endif
 
-#if defined(VMS) || defined(VMCMS) || defined(DOS) || defined(OS2) || defined(__EMX__) || defined(WIN32) || defined(__DJGPP__) || defined(__CYGWIN32__) || defined(MACOS)
+#if defined(VMS) || defined(VMCMS) || defined(DOS) || defined(OS2) || defined(__EMX__) || defined(WIN32) || defined(__DJGPP__) || defined(MACOS)
 #define POPEN_READ_MODE "rb"
 #else
 #define POPEN_READ_MODE "r"
 #endif
 
-//------------------------------------------------------------------------
-// uncompress program
-//------------------------------------------------------------------------
-
-#ifdef HAVE_POPEN
-
-// command to uncompress to stdout
-#  ifdef USE_GZIP
-#    define uncompressCmd "gzip -d -c -q"
-#  else
-#    ifdef __EMX__
-#      define uncompressCmd "compress -d -c"
-#    else
-#      define uncompressCmd "uncompress -c"
-#    endif // __EMX__
-#  endif // USE_GZIP
-
-#else // HAVE_POPEN
-
-// command to uncompress a file
-#  ifdef USE_GZIP
-#    define uncompressCmd "gzip -d -q"
-#  else
-#    define uncompressCmd "uncompress"
-#  endif // USE_GZIP
-
-#endif // HAVE_POPEN
-
 //------------------------------------------------------------------------
 // Win32 stuff
 //------------------------------------------------------------------------
@@ -121,7 +97,7 @@
 #undef CDECL
 #endif
 
-#ifdef _MSC_VER
+#if defined(_MSC_VER) || defined(__BORLANDC__)
 #define CDECL __cdecl
 #else
 #define CDECL
