@@ -2,7 +2,7 @@
 //
 // pdffonts.cc
 //
-// Copyright 2001-2003 Glyph & Cog, LLC
+// Copyright 2001-2007 Glyph & Cog, LLC
 //
 //========================================================================
 
@@ -12,6 +12,7 @@
 #include <stddef.h>
 #include <string.h>
 #include <math.h>
+#include <limits.h>
 #include "parseargs.h"
 #include "GString.h"
 #include "gmem.h"
@@ -24,15 +25,20 @@
 #include "PDFDoc.h"
 #include "config.h"
 
-static char *fontTypeNames[] = {
+// NB: this must match the definition of GfxFontType in GfxFont.h.
+static const char *fontTypeNames[] = {
   "unknown",
   "Type 1",
   "Type 1C",
+  "Type 1C (OT)",
   "Type 3",
   "TrueType",
+  "TrueType (OT)",
   "CID Type 0",
   "CID Type 0C",
-  "CID TrueType"
+  "CID Type 0C (OT)",
+  "CID TrueType",
+  "CID TrueType (OT)"
 };
 
 static void scanFonts(Dict *resDict, PDFDoc *doc);
@@ -135,8 +141,8 @@ int main(int argc, char *argv[]) {
   }
 
   // scan the fonts
-  printf("name                                 type         emb sub uni object ID\n");
-  printf("------------------------------------ ------------ --- --- --- ---------\n");
+  printf("name                                 type              emb sub uni object ID\n");
+  printf("------------------------------------ ----------------- --- --- --- ---------\n");
   fonts = NULL;
   fontsLen = fontsSize = 0;
   for (pg = firstPage; pg <= lastPage; ++pg) {
@@ -144,8 +150,7 @@ int main(int argc, char *argv[]) {
     if ((resDict = page->getResourceDict())) {
       scanFonts(resDict, doc);
     }
-    annots = new Annots(doc->getXRef(), doc->getCatalog(),
-			page->getAnnots(&obj1));
+    annots = new Annots(doc, page->getAnnots(&obj1));
     obj1.free();
     for (i = 0; i < annots->getNumAnnots(); ++i) {
       if (annots->getAnnot(i)->getAppearance(&obj1)->isStream()) {
@@ -242,7 +247,7 @@ static void scanFont(GfxFont *font, PDFDoc *doc) {
   }
 
   // font name
-  name = font->getOrigName();
+  name = font->getName();
 
   // check for an embedded font
   if (font->getType() == fontType3) {
@@ -272,7 +277,7 @@ static void scanFont(GfxFont *font, PDFDoc *doc) {
   }
 
   // print the font info
-  printf("%-36s %-12s %-3s %-3s %-3s",
+  printf("%-36s %-17s %-3s %-3s %-3s",
 	 name ? name->getCString() : "[none]",
 	 fontTypeNames[font->getType()],
 	 emb ? "yes" : "no",
@@ -286,7 +291,12 @@ static void scanFont(GfxFont *font, PDFDoc *doc) {
 
   // add this font to the list
   if (fontsLen == fontsSize) {
-    fontsSize += 32;
+    if (fontsSize <= INT_MAX - 32) {
+      fontsSize += 32;
+    } else {
+      // let greallocn throw an exception
+      fontsSize = -1;
+    }
     fonts = (Ref *)greallocn(fonts, fontsSize, sizeof(Ref));
   }
   fonts[fontsLen++] = *font->getID();
