@@ -29,13 +29,16 @@
 #include "Error.h"
 #include "config.h"
 
-static void printInfoString(FILE *f, Dict *infoDict, char *key,
-			    char *text1, char *text2, UnicodeMap *uMap);
-static void printInfoDate(FILE *f, Dict *infoDict, char *key, char *fmt);
+static void printInfoString(FILE *f, Dict *infoDict, const char *key,
+			    const char *text1, const char *text2,
+			    UnicodeMap *uMap);
+static void printInfoDate(FILE *f, Dict *infoDict, const char *key,
+			  const char *fmt);
 
 static int firstPage = 1;
 static int lastPage = 0;
 static GBool physLayout = gFalse;
+static double fixedPitch = 0;
 static GBool rawOrder = gFalse;
 static GBool htmlMeta = gFalse;
 static char textEncName[128] = "";
@@ -55,6 +58,8 @@ static ArgDesc argDesc[] = {
    "last page to convert"},
   {"-layout",  argFlag,     &physLayout,    0,
    "maintain original physical layout"},
+  {"-fixed",   argFP,       &fixedPitch,    0,
+   "assume fixed-pitch (or tabular) text"},
   {"-raw",     argFlag,     &rawOrder,      0,
    "keep strings in content stream order"},
   {"-htmlmeta", argFlag,   &htmlMeta,       0,
@@ -112,6 +117,9 @@ int main(int argc, char *argv[]) {
     goto err0;
   }
   fileName = new GString(argv[1]);
+  if (fixedPitch) {
+    physLayout = gTrue;
+  }
 
   // read config file
   globalParams = new GlobalParams(cfgFileName);
@@ -132,7 +140,7 @@ int main(int argc, char *argv[]) {
 
   // get mapping to output encoding
   if (!(uMap = globalParams->getTextEncoding())) {
-    error(-1, "Couldn't get text encoding");
+    error(errConfig, -1, "Couldn't get text encoding");
     delete fileName;
     goto err1;
   }
@@ -162,7 +170,8 @@ int main(int argc, char *argv[]) {
 
   // check for copy permission
   if (!doc->okToCopy()) {
-    error(-1, "Copying of text from this document is not allowed.");
+    error(errNotAllowed, -1,
+	  "Copying of text from this document is not allowed.");
     exitCode = 3;
     goto err2;
   }
@@ -195,7 +204,7 @@ int main(int argc, char *argv[]) {
       f = stdout;
     } else {
       if (!(f = fopen(textFileName->getCString(), "wb"))) {
-	error(-1, "Couldn't open text file '%s'", textFileName->getCString());
+	error(errIO, -1, "Couldn't open text file '{0:t}'", textFileName);
 	exitCode = 2;
 	goto err3;
       }
@@ -232,7 +241,7 @@ int main(int argc, char *argv[]) {
 
   // write text file
   textOut = new TextOutputDev(textFileName->getCString(),
-			      physLayout, rawOrder, htmlMeta);
+			      physLayout, fixedPitch, rawOrder, htmlMeta);
   if (textOut->isOk()) {
     doc->displayPages(textOut, firstPage, lastPage, 72, 72, 0,
 		      gFalse, gTrue, gFalse);
@@ -249,7 +258,7 @@ int main(int argc, char *argv[]) {
       f = stdout;
     } else {
       if (!(f = fopen(textFileName->getCString(), "ab"))) {
-	error(-1, "Couldn't open text file '%s'", textFileName->getCString());
+	error(errIO, -1, "Couldn't open text file '{0:t}'", textFileName);
 	exitCode = 2;
 	goto err3;
       }
@@ -281,8 +290,9 @@ int main(int argc, char *argv[]) {
   return exitCode;
 }
 
-static void printInfoString(FILE *f, Dict *infoDict, char *key,
-			    char *text1, char *text2, UnicodeMap *uMap) {
+static void printInfoString(FILE *f, Dict *infoDict, const char *key,
+			    const char *text1, const char *text2,
+			    UnicodeMap *uMap) {
   Object obj;
   GString *s1;
   GBool isUnicode;
@@ -318,7 +328,8 @@ static void printInfoString(FILE *f, Dict *infoDict, char *key,
   obj.free();
 }
 
-static void printInfoDate(FILE *f, Dict *infoDict, char *key, char *fmt) {
+static void printInfoDate(FILE *f, Dict *infoDict, const char *key,
+			  const char *fmt) {
   Object obj;
   char *s;
 
